#*******************************************************************************#
#                                                                               #
#                            E A S I N S T A L L E R                            #
#                                                                               #
# Application Toolkit: Application class                                        #
#*******************************************************************************#
# Copyright (C) 2004 by Yuri Leikind.                                           #
#*******************************************************************************#
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public                           #
# License as published by the Free Software Foundation                          #
#                                                                               #
# This program is distributed in the hope that it will be useful,               #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             #
# General Public License for more details.                                      #
#                                                                               #
# You should have received a copy of the GNU General Public                     #
# License along with this program; if not, write to the Free Software           #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    #
#*******************************************************************************#
# $Id: application.rb,v 1.6 2004/07/09 16:01:38 leikind Exp $                   #
#*******************************************************************************#

require "app_toolkit/compatibility.rb"
require "singleton.rb"
require "app_toolkit/signals.rb"
require "app_toolkit/fox_extensions.rb"
require "app_toolkit/configuration.rb"
require "app_toolkit/resources.rb"
require "app_toolkit/dialog.rb"
require "app_toolkit/message.rb"
require "app_toolkit/menubar_toolbar.rb"
require "app_toolkit/main_window.rb"


module AppToolkit

    BUTTON_STYLE = Fox::FRAME_SUNKEN
    FRAME_STYLE  = Fox::FRAME_SUNKEN

    class Application < Fox::FXApp

        include Fox

        attr_reader :window, :config, :resources, :icons, :title, :locale
        
        def initialize(
                app_name,
                resource_path,
                table_path,
                vendor_name = "Vendor",
                prerun_action = nil, 
                close_action = nil,
                config_dir_name = nil,
                config_filename = nil
                )

            super(app_name, vendor_name)
            @title = app_name
            @prerun_action = prerun_action
            @close_action = close_action
            
            if config_dir_name.nil?
                config_dir_name =  "." + app_name.gsub(/[\s]/, "_") 
            end
            
            if config_filename.nil?
                config_filename = "config.yaml"
            end
            
            @icons = Icons.new(self)
            @config = Config.new(config_dir_name, config_filename)
            
            @resources = Resources.instance
            @resources.path = resource_path
            
            Fox.application_resources = @resources 
            
            table_path = "resources/resource_table.yaml"
            @locale = Locale.new(table_path, @config, @resources)    
            
            self.normalFont = @locale.get_font(self)
            FXTooltip.new(self, TOOLTIP_PERMANENT)
        end
        
        def set_window(window)
            @window = window
            @window.connect(SEL_CLOSE){
                @close_action.call if @close_action.class == Proc
                exit
            }            
        end

        def run_()
            create
            @window.show
            @prerun_action.call if @prerun_action.class == Proc 
            run
        end
    end
end


