#*******************************************************************************#
#                                                                               #
#                            E A S I N S T A L L E R                            #
#                                                                               #
# Application Toolkit: Dialog class                                             #
#*******************************************************************************#
# Copyright (C) 2004 by Yuri Leikind.                                           #
#*******************************************************************************#
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public                           #
# License as published by the Free Software Foundation                          #
#                                                                               #
# This program is distributed in the hope that it will be useful,               #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             #
# General Public License for more details.                                      #
#                                                                               #
# You should have received a copy of the GNU General Public                     #
# License along with this program; if not, write to the Free Software           #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    #
#*******************************************************************************#
# $Id: dialog.rb,v 1.4 2004/07/06 15:23:37 leikind Exp $                        #
#*******************************************************************************#

module AppToolkit
    class Dialog < Fox::FXDialogBox

        include Fox
        
        OK = "ok"
        APPLY = "apply"
        CANCEL = "cancel"

        def initialize(parent, title_key, x, y, procs, horizontal = false)
            @resources = AppToolkit::Resources.instance
        
            caption = @resources.has_key?(title_key) ? 
                @resources[title_key].sub(/\&/,"") : title_key
            
            super(parent, caption, DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE, 0, 0, 
                x, y, 0, 0, 0, 0, 4, 4)

            vertical = FXVerticalFrame.new(self,
                AppToolkit::FRAME_STYLE|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y)

            content_type = horizontal ? FXHorizontalFrame : FXVerticalFrame
            @content = content_type.new(vertical,
                Fox::FRAME_NONE|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y)

            @button_container = FXHorizontalFrame.new(vertical,
                FRAME_NONE|LAYOUT_SIDE_TOP|LAYOUT_FILL_X, 0,0,0,0, 5,5,5,5, 7,7)
                
            @buttons = Hash.new
            [OK, APPLY, CANCEL].each do |proc_type|
                if procs.has_key?(proc_type)
                    build_button(proc_type, procs[proc_type])
                end
            end
        end
        
        def validate
            true
        end
        
        def build_button(res_key, proc_obj)
            button = FXButton.new(@button_container, @resources[res_key], nil, nil, 0,
                LAYOUT_FILL_Y|LAYOUT_CENTER_Y|LAYOUT_FILL_X|AppToolkit::BUTTON_STYLE)
                
            @buttons[res_key] = button

            button.easinstaller_text_key = { "text=" => res_key }  

            if res_key == OK
                button.setFocus
            end

            button.connect(SEL_COMMAND) do
                proc_obj.call
                if res_key == APPLY
                    validate()
                end                
                if res_key == OK
                    if validate
                        self.handle(button, MKUINT(FXDialogBox::ID_ACCEPT, SEL_COMMAND), nil )
                    end
                end
                if res_key == CANCEL
                    self.handle(button, MKUINT(FXDialogBox::ID_CANCEL, SEL_COMMAND), nil )
                end                
            end
        end
    end
end

