#*******************************************************************************#
#                                                                               #
#                            E A S I N S T A L L E R                            #
#                                                                               #
# Application Toolkit: FoX extension code                                       #
#*******************************************************************************#
# Copyright (C) 2004 by Yuri Leikind.                                           #
#*******************************************************************************#
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public                           #
# License as published by the Free Software Foundation                          #
#                                                                               #
# This program is distributed in the hope that it will be useful,               #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             #
# General Public License for more details.                                      #
#                                                                               #
# You should have received a copy of the GNU General Public                     #
# License along with this program; if not, write to the Free Software           #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    #
#*******************************************************************************#
# $Id: fox_extensions.rb,v 1.3 2004/07/06 15:23:37 leikind Exp $                #
#*******************************************************************************#

module Fox
    module Disabling
        def set_disabling(signal_start, signal_end)
            hook(signal_start){
                @status = self.enabled
                self.enabled = false
            }

            hook(signal_end){
                self.enabled = @status
            }
        end
    end

    class FXMenuCommand
        include Disabling
    end

    class FXLabel
        include Disabling
    end

    class FXObject
        attr_accessor :easinstaller_text_key
    end

    def self.each_fx_label(classes)
        ObjectSpace.each_object{|obj|
            classes.each{|klass|
                if obj.kind_of? klass
                    # puts "yielding object #{obj.class}"
                    begin
                        yield obj
                    rescue RuntimeError
                        # a workaround for 
                        # "This FXLabel * already released"
                        # error
                        
                        #puts $!.class, $!.message
                    end
                end
            }
        }
    end

    def self.update_font(new_font)
        each_fx_label([FXLabel, FXTooltip, FXStatusline, FXMenuCommand,
            FXMenuTitle, FXGroupBox, FXComboBox]){|obj|
            obj.font = new_font
        }
    end

    
    def self.application_resources=(res)
        @@app_resources = res
    end
    
    def self.update_text()
        each_fx_label([FXLabel, FXStatusline, FXMenuCommand, FXMenuTitle,
                        FXGroupBox, FXComboBox]){|obj|
            if obj.respond_to? :easinstaller_text_key
                if obj.easinstaller_text_key.class == Hash
                    obj.easinstaller_text_key.each {|method, value|
                        if value.class == Array  # FXComboBox hack
                            value.each_with_index{|v, i|
                                obj.send(method, i, @@app_resources[v])
                            }
                        else
                            obj.send(method, @@app_resources[value])
                        end
                    }
                end
            end
        }
    end
end
