#*******************************************************************************#
#                                                                               #
#                            E A S I N S T A L L E R                            #
#                                                                               #
# Application Toolkit: Common menu and toolbar API                              #
#*******************************************************************************#
# Copyright (C) 2004 by Yuri Leikind.                                           #
#*******************************************************************************#
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public                           #
# License as published by the Free Software Foundation                          #
#                                                                               #
# This program is distributed in the hope that it will be useful,               #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             #
# General Public License for more details.                                      #
#                                                                               #
# You should have received a copy of the GNU General Public                     #
# License along with this program; if not, write to the Free Software           #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    #
#*******************************************************************************#
# $Id: menubar_toolbar.rb,v 1.5 2004/07/09 16:01:38 leikind Exp $               #
#*******************************************************************************#

module AppToolkit
    class ToolbarButton < Fox::FXButton
        include Fox
        def initialize(parent, title, status_msg, tooltip_msg, icon)
            #name = form_name(title, status_msg, tooltip_msg)
            super(parent, title, icon, nil, 0,
                ICON_ABOVE_TEXT|BUTTON_TOOLBAR|BUTTON_STYLE)
            self.helpText = tooltip_msg
            self.tipText = status_msg
        end
    end
    
    
    class MenubarAndToolbar
        include Fox

        def initialize(parent, resources, icons)
            @resources = resources
            @icons = icons
            dragshell = FXToolbarShell.new(parent, FRAME_RAISED|FRAME_THICK)
            
            @menubar = FXMenubar.new(parent, dragshell, LAYOUT_SIDE_TOP|LAYOUT_FILL_X)
            FXToolbarGrip.new(@menubar, @menubar, FXMenubar::ID_TOOLBARGRIP,  TOOLBARGRIP_DOUBLE)

            dragshell = FXToolbarShell.new(parent, FRAME_RAISED|FRAME_THICK)
            @toolbar = FXToolbar.new(parent, dragshell,
               LAYOUT_SIDE_TOP|PACK_UNIFORM_WIDTH|PACK_UNIFORM_HEIGHT|FRAME_RAISED|LAYOUT_FILL_X)
            FXToolbarGrip.new(@toolbar, @toolbar, FXMenubar::ID_TOOLBARGRIP, TOOLBARGRIP_DOUBLE)

            @menupanes = Hash.new
        end

        def menupane(title, orientation)
            @menupanes[title] = FXMenuPane.new(@menubar)

            menu_title = FXMenuTitle.new(@menubar, @resources[title],  nil,
                @menupanes[title], orientation)

            menu_title.easinstaller_text_key = {
                "text=" => title
            }
        end

        def action(data)

            initial_resource_key = {
                "text="     => data["title"].dup,
                "tipText="  => data["status message"].dup,
                "helpText=" => data["tooltip message"].dup
            }

            initial_resource_key_menu = {
                "text="     => data["title"].dup,
                "helpText=" => data["tooltip message"].dup
            }

            data["title"] = @resources[data["title"]]
            data["status message"] = @resources[data["status message"]]
            data["tooltip message"] = @resources[data["tooltip message"]]

            menu_command = FXMenuCommand.new(@menupanes[data["menupane"]], data["title"], nil)
            menu_command.helpText = data["tooltip message"]
            menu_command.easinstaller_text_key = initial_resource_key_menu

            process_common(menu_command, data)

            if data["both"]
                icon = nil
                if data.has_key?("icon")
                    icon = @icons.get_icon(data["icon"])
                end

                button = ToolbarButton.new(@toolbar, data["title"],
                    data["status message"], data["tooltip message"], icon)

                button.easinstaller_text_key = initial_resource_key

                process_common(button, data)
            end
        end

        private

        def process_common(obj, data)
            set_action(obj, data["action"])

            if data["disable between signals"].class == Array
                set_disabling(obj, data["disable between signals"])
            end

            if data["disabled initially"]
                obj.enabled = false
            end

            if data["disable on"].class == Class
                hook(data["disable on"]){
                    obj.enabled = false
                }
            end

            if data["enable on signal"]
                hook(data["enable on signal"]){
                    obj.enabled = true
                }
            end
        end

        def set_disabling(obj, signals)
            obj.set_disabling(signals[0], signals[1])
        end

        def set_action(obj, action)
            obj.connect(SEL_COMMAND){
                action[0].send(action[1])
            }
        end

     end    
end    

