#*******************************************************************************#
#                                                                               #
#                            E A S I N S T A L L E R                            #
#                                                                               #
# Application Toolkit: Resources code                                           #
#*******************************************************************************#
# Copyright (C) 2004 by Yuri Leikind.                                           #
#*******************************************************************************#
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public                           #
# License as published by the Free Software Foundation                          #
#                                                                               #
# This program is distributed in the hope that it will be useful,               #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             #
# General Public License for more details.                                      #
#                                                                               #
# You should have received a copy of the GNU General Public                     #
# License along with this program; if not, write to the Free Software           #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    #
#*******************************************************************************#
# $Id: resources.rb,v 1.7 2004/07/22 12:32:55 leikind Exp $                     #
#*******************************************************************************#

require "fox"
require "singleton.rb"

module AppToolkit

    class Locale

        DEFAULT_LOCALE = "en_US"

        def initialize(table_path, config, resources)
            @table = exclude_invalid_entries(YAML::load(File.open(table_path)))
            @config = config
            @resources = resources
            @locale = @config["locale"]
            if @locale.nil?
                @locale = DEFAULT_LOCALE
                @config["locale"] = DEFAULT_LOCALE
            end
            @resources.load(@locale)
        end
        
        def exclude_invalid_entries(table)
            table.reject{|key, a_table|
                if a_table["invalid_constraint"]
                    eval(a_table["invalid_constraint"])
                else
                    false
                end
            }        
        end

        def current_locale
            @locale
        end

        def set_locale(locale)
            unless @table.has_key?(locale)
                raise "Attempt to set a locale that has not been registered!"
            end
            @locale = locale
            @config["locale"] = locale
            @resources.load(locale)
        end

        include Fox

        def lang_list
            @table.collect{|key, a_table|
                [key, a_table["language_name"]]
            }
        end

        def save_font(font)
            fontdesc = font.fontDesc

            fontdesc_characteristics = {
                "face" => fontdesc.face,
                "encoding" => fontdesc.encoding,
                "size" => fontdesc.size,
                "slant" => fontdesc.slant,
                "weight" => fontdesc.weight,
                "setwidth" => fontdesc.setwidth,
                "flags" => fontdesc.flags
            }
            @config["font"] = fontdesc_characteristics
            @predefined = false
        end

        def set_predefined
            @config.delete("font")
            @predefined = true
        end

        def get_default_font_table
            font_info = @table[@locale]
            if font_info.nil?
                STDERR.puts "Font description for locale #{@locale} is not" +
                    " found, falling back to default font"
                font_info = @table[DEFAULT_LOCALE]
            end
            return font_info
        end

        def get_default_font(app)
            font_info = get_default_font_table()
            return restore_font(app, font_info)
        end

        def using_predefined_font?
            @predefined
        end

        def get_font(app, loc = @locale)

            font_info = nil

            if @config["font"]
                log "config font"
                font_info = @config["font"]
                @predefined = false
            else
                log "predefined font"
                @predefined = true
                font_info = get_default_font_table()
            end
            return restore_font(app, font_info)
        end

        def restore_font(app, font_info)
            fontdescr = FXFontDesc.new()
            
            if font_info["encoding"].class == Fixnum
                fontdescr.encoding = font_info["encoding"].to_i
            else
                fontdescr.encoding = eval(font_info["encoding"])
            end

            if font_info["slant"].class == Fixnum
                fontdescr.slant = font_info["slant"].to_i
            else
                fontdescr.slant = eval(font_info["slant"])
            end


            if font_info["weight"].class == Fixnum
                fontdescr.weight = font_info["weight"].to_i
            else
                fontdescr.weight = eval(font_info["weight"])
            end

            if font_info["setwidth"].class == Fixnum
                fontdescr.setwidth = font_info["setwidth"].to_i
            else
                fontdescr.setwidth = eval(font_info["setwidth"])
            end


            fontdescr.size = font_info["size"].to_i
            fontdescr.face = font_info["face"]
            fontdescr.flags = font_info["flags"].to_i

            a = FXFont.new(app, fontdescr)
            a.create()
            return a
        end
    end

    
    class Resources
        
        include Singleton 
        
        attr_writer :path

        def load(lang)
            @lang = lang
            filename = @path + @lang
            unless FileTest.exists?(filename)
                STDERR.puts "Resource file #{filename} for locale #{lang} is not" +
                " found, falling back on default #{Locale::DEFAULT_LOCALE}"
                filename = PATH + Locale::DEFAULT_LOCALE
            end
            @key_offsets = Hash.new
            @file = File.new(filename)
            build_keys()
        end
        
        def build_keys()
            while line = @file.gets
                if line =~ /^(([\w\d_-]+)\s*:\s+)"((.+?)"\s*)$/
                    key_holder, key, value_holder, value = $1, $2, $3, $4
                    @key_offsets[key] = [@file.pos - value_holder.size, value.size]
                else
                    #puts "could not parse line #{@file.lineno}:#{line.dump}"
                end
            end
        end
        

        def [](key)
            if @key_offsets.nil?
                raise "Need to call #load(lang) first"
            end
            if @key_offsets.has_key?(key)
                offset, size = @key_offsets[key]
                @file.seek(offset)
                return eval('"' + @file.read(size) + '"') 
            else
                raise "Resource error: No such message for key #{key} with language #{@lang}"
            end
        end
        
        def has_key?(key)
            @key_offsets.has_key?(key)
        end
    end
    
    
    class Icons
        def initialize(app)
            @icons = Hash.new
            @app = app
        end

        def get_icon(path)
            unless @icons.has_key?(path)
                if FileTest.exists?(path)
                    File.open(path, "rb"){|f|
                        @icons[path] = Fox::FXPNGIcon.new(@app, f.read)
                    }
                else
                    return nil
                end
            end
            @icons[path]
        end
    end
    
end

if $0 == __FILE__
    resources = AppToolkit::Resources.instance("resources/resources.")
    resources.path = "resources/resources."
    resources.load("en_US")
    
    puts resources["is_missing"]
    #config = AppToolkit::Config.new(".easinstaller", "config")
    #table_path = "resources/resource_table.yaml"
    #locale = AppToolkit::Locale.new(table_path, config, resources)    
end

