#*******************************************************************************#
#                                                                               #
#                            E A S I N S T A L L E R                            #
#                                                                               #
# Application Toolkit: Application signalling infrastructure                    #
#*******************************************************************************#
# Copyright (C) 2004 by Yuri Leikind.                                           #
#*******************************************************************************#
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public                           #
# License as published by the Free Software Foundation                          #
#                                                                               #
# This program is distributed in the hope that it will be useful,               #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             #
# General Public License for more details.                                      #
#                                                                               #
# You should have received a copy of the GNU General Public                     #
# License along with this program; if not, write to the Free Software           #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    #
#*******************************************************************************#
# $Id: signals.rb,v 1.4 2004/07/06 15:23:37 leikind Exp $                       #
#*******************************************************************************#


module AppToolkit
    module Signals
        class BaseSignal; end
    end
end

class Object
    @@hooks = Array.new

    def notify(signal, *args)
        @@hooks.each do |hook|
            receiver, signal_candidate, block = hook
            if signal.ancestors.index(signal_candidate)
                block.call(self, *args)
            end
        end
    end


    def request(signal, *args)
        res = nil
        @@hooks.each do |hook|
            receiver, signal_candidate, block = hook
            if signal.ancestors.index(signal_candidate)
                res = block.call(self, *args)
                break
            end
        end
        if res.nil?
            raise "Request processor for signal #{signal} not found"
        end
        res
    end


    def hook(signal, &proc)
        @@hooks << [self, signal, proc]
        proc
    end

    # not really related to the signal system, but...
    def safe_send(method, *args)
        self.send(method, *args) if self.respond_to?(method)
    end

    DEBUG = true

    def log(str)
        puts str if DEBUG
    end
end



