#*******************************************************************************#
#                                                                               #
#                            E A S I N S T A L L E R                            #
#                                                                               #
# Archive handling                                                              #
#*******************************************************************************#
# Copyright (C) 2004 by Yuri Leikind.                                           #
#*******************************************************************************#
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public                           #
# License as published by the Free Software Foundation                          #
#                                                                               #
# This program is distributed in the hope that it will be useful,               #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             #
# General Public License for more details.                                      #
#                                                                               #
# You should have received a copy of the GNU General Public                     #
# License along with this program; if not, write to the Free Software           #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    #
#*******************************************************************************#
# $Id: archives.rb,v 1.9 2004/07/07 15:25:05 leikind Exp $                      #
#*******************************************************************************#


module Core

    class Archive
        attr_accessor  :target_directory

        def initialize(f)
            @filename = f
        end


        @@masks = {
            /\.tar$/i  => :tar,
            /\.tar\.gz$/i  => :tar_gz,
            /\.tgz$/i  => :tar_gz,
            /\.tar\.bz2$/i  => :tar_bz2,
            /\.zip$/i  => :zip
        }

        @@unpack_commands = {
            :tar => "tar -xvf _FILE_ -C _DIR_",
            :tar_gz =>  "gunzip  -c _FILE_ | tar -xv -C _DIR_",
            :tar_bz2 => "bunzip2 -c _FILE_ | tar -xv -C _DIR_",
            :zip => "unzip _FILE_ -d _DIR_"
        }

        @@verify_commands = {
            :tar_gz =>  "gzip -t _FILE_",
            :tar_bz2 => "bzip2 -t _FILE_",
            :zip => "zip -T _FILE_"
        }


        def unpack
            command = @@unpack_commands[@type].sub(Regexp.new("_FILE_"), @filename).
                                               sub(Regexp.new("_DIR_"), @target_directory)

            command = Command.new(command)
            resources = AppToolkit::Resources.instance
            ipc = Core::IPC.instance
            res = ipc.system(command, resources["unpacking1"] + 
                @type.to_s.sub(/_/,".") + resources["unpacking2"])
                

            entries = Dir.entries(@target_directory).reject{|d| d == "." or d == ".."}
            if entries.length == 1
                @src_directory = @target_directory.sub(/\/$/,"") + "/" + entries[0]
            else
                mess = "more than one directory in the target directory"
                raise mess
            end
            
            if res
                return @src_directory
            else
                return [@src_directory]
            end
        end

        class InvalidArchive < Exception ; end

        def verify_archive
            if @@verify_commands.has_key?(@type)
                command = @@verify_commands[@type].sub(Regexp.new("_FILE_"), @filename) +
                    " > /dev/null 2> /dev/null"
                unless system(command)
                    raise InvalidArchive
                end
            end
        end

        def recognize_filetype
            @@masks.each {|regex, type|
                if regex.match(@filename)
                    @type = type
                    verify_archive
                    return @type
                end
            }
            return false
        end

    end
end
