#*******************************************************************************#
#                                                                               #
#                            E A S I N S T A L L E R                            #
#                                                                               #
#  Menu, toolbar, hooks                                                         #
#*******************************************************************************#
# Copyright (C) 2004 by Yuri Leikind.                                           #
#*******************************************************************************#
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public                           #
# License as published by the Free Software Foundation                          #
#                                                                               #
# This program is distributed in the hope that it will be useful,               #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             #
# General Public License for more details.                                      #
#                                                                               #
# You should have received a copy of the GNU General Public                     #
# License along with this program; if not, write to the Free Software           #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    #
#*******************************************************************************#
# $Id: actions.rb,v 1.35 2004/07/07 15:25:05 leikind Exp $                      #
#*******************************************************************************#

module UI
    class TopWindow  
        def make_menu_and_toolbar()
            #menu_and_toolbar = MenubarAndToolbar.new(self)

            self.extend(MenuAndToolbarHooks)

            mt.menupane("file", LAYOUT_LEFT)
            mt.menupane("edit", LAYOUT_LEFT)
            mt.menupane("actions", LAYOUT_LEFT)
            mt.menupane("help", LAYOUT_RIGHT)
           
            mt.action( "title" => "open",
                "icon" => "icons/open.png",
                "status message" => "open_explained2",
                "tooltip message" => "open_explained",
                "action" => [self, :open_trigger],
                "menupane" => "file",
                "both" => true,
                "disable between signals" => 
                    [UI::Signals::NewCommand, UI::Signals::CommandFinished]
            )   
    
            mt.action( "title" => "open_directory",
                "icon" => "icons/open_directory.png",
                "status message" => "open_directory_explained2",
                "tooltip message" => "open_directory_explained",
                "action" => [self, :open_directory_trigger],
                "menupane" => "file",
                "both" => true,
                "disable between signals" => 
                    [UI::Signals::NewCommand, UI::Signals::CommandFinished]
            )


            mt.action( "title" => "project_list",
                "status message" => "project_list_explained",
                "tooltip message" => "project_list_explained",
                "action" => [self, :session_list_trigger],
                "menupane" => "edit",
                "both" => false,
                "disable between signals" => 
                    [UI::Signals::NewCommand, UI::Signals::CommandFinished]
            )


            mt.action( "title" => "project_settings",
                "status message" => "project_settings_explained",
                "tooltip message" => "project_settings_explained",
                "action" => [self, :settings_trigger],
                "menupane" => "edit",
                "both" => false,
                "disable between signals" => 
                    [UI::Signals::NewCommand, UI::Signals::CommandFinished],
                "disabled initially" => true ,
                "enable on signal" => UI::Signals::NewSessionStarted
            )


            mt.action( "title" => "settings",
                "status message" => "program_settings_explained",
                "tooltip message" => "program_settings_explained2",
                "action" => [self, :program_settings_trigger],
                "menupane" => "edit",
                "both" => false,
                "disable between signals" => 
                     [UI::Signals::NewCommand, UI::Signals::CommandFinished],
                "disabled initially" => false
            )


            mt.action( "title" => "quit",
                "status message" => "quit_explained",
                "tooltip message" => "quit_explained",
                "action" => [self, :quit_trigger],
                "menupane" => "file",
                "both" => false,
                "disable between signals" => false
            )

   
            mt.action( "title" => "configure_build",
               "icon" => "icons/configure.png",
                "status message" => "configure_build_explained2",
                "tooltip message" => "configure_build_explained",
                "action" => [self, :configure_trigger],
                "menupane" => "actions",
                "both" => true,
                "disable between signals" => 
                    [UI::Signals::NewCommand, UI::Signals::CommandFinished],
                "disable on" => UI::Signals::NewSessionStarted,
                "disabled initially" => true,
                "enable on signal" => UI::Signals::OptionsParsed
            )

            mt.action( "title" => "build",
                "icon" => "icons/build.png",
                "status message" => "build_explained2",
                "tooltip message" => "build_explained",
                "action" => [self, :build_trigger],
                "menupane" => "actions",
                "both" => true,
                "disable between signals" => 
                    [UI::Signals::NewCommand, UI::Signals::CommandFinished],
                "disable on" => UI::Signals::NewSessionStarted,
                "disabled initially" => true,
                "enable on signal" => UI::Signals::ConfigureFinished
            )

            mt.action( "title" => "install",
                "icon" => "icons/install.png",
                "status message" => "install_explained2",
                "tooltip message" => "install_explained",
                "action" => [self, :install_trigger],
                "menupane" => "actions",
                "both" => true,
                "disable between signals" => 
                    [UI::Signals::NewCommand, UI::Signals::CommandFinished],
                "disable on" => UI::Signals::NewSessionStarted,
                "disabled initially" => true,
                "enable on signal" => UI::Signals::ConfigureFinished
            )

            mt.action( "title" => "about",
                "status message" => "about_explained",
                "tooltip message" => "about_explained",
                "action" => [self, :about_trigger],
                "menupane" => "help",
                "both" => false,
                "disable between signals" => false
            )
  
        end
    end

    module MenuAndToolbarHooks
    
        OPEN_MASK = "All Archives (*.tar,*.gz,*.tgz,*.bz2,*.zip)\n" +
                    "Tar Archives (*.tar)\n" +
                    "Gzip Archives (*.gz,*.tgz)\n" +
                    "Bzip2 Archives (*.bz2)\n" +
                    "Zip Archives (*.zip)\n" +
                    "All Files (*)\n"


        def open_directory_trigger(dir = nil)
            resources = AppToolkit::Resources.instance
        
            if dir.nil?
                last_dir = UI::EasinstallerApp.get_app.config["last_dir2"] || ENV["HOME"]+"/"
                dir = Fox::FXFileDialog.getOpenDirectory(EasinstallerApp.get_top_window,
                    resources["open_source_directory"], last_dir)
            end

            unless dir.empty? or dir.nil?
                UI::EasinstallerApp.get_app.config["last_dir2"] = dir

                notify(UI::Signals::NewSessionPossible)
                session = Core::Session.new_from_directory(dir)

                Thread.new{
                    UI::error_wrapper{
                        if session.configure_present?
                            EasinstallerApp.get_top_window.session = session
                            session.run
                            EasinstallerApp.get_top_window.title = session.name
                        elsif session.autogen_present?()
                            AppToolkit::Message.new(EasinstallerApp.get_top_window, AppToolkit::Message::ERROR,
                                resources["autogen1"],
                                resources["autogen2"]).execute
                            session.run_autogen
                            if session.configure_present?()
                                AppToolkit::Message.new(EasinstallerApp.get_top_window, AppToolkit::Message::ERROR,
                                    resources["autogen_success1"],
                                    resources["autogen_success2"]).execute
                                EasinstallerApp.get_top_window.session = session
                                EasinstallerApp.get_top_window.title = session.name
                                session.run
                            else
                                AppToolkit::Message.new(EasinstallerApp.get_top_window, AppToolkit::Message::ERROR,
                                    resources["autogen_failure1"],
                                    resources["autogen_failure2"]).execute
                            end
                        elsif session.makefile_present?()
                            AppToolkit::Message.new(EasinstallerApp.get_top_window, AppToolkit::Message::ERROR,
                                resources["only_makefile1"],
                                resources["only_makefile2"]).execute
                            EasinstallerApp.get_top_window.session = session
                            EasinstallerApp.get_top_window.title = session.name
                            session.run(false)
                        else
                            AppToolkit::Message.new(EasinstallerApp.get_top_window, AppToolkit::Message::ERROR,
                                resources["unsupported_package1"],
                                resources["unsupported_package2"]).execute

                        end
                    }
                }
            end
        end


        def open_trigger(file = nil)
            
            resources = AppToolkit::Resources.instance

            if file.nil?
                dir = UI::EasinstallerApp.get_app.config["last_dir"] || ENV["HOME"]+"/"
                file = Fox::FXFileDialog.getOpenFilename(EasinstallerApp.get_top_window, 
                    resources["open_file"],  dir, OPEN_MASK, 0)
            end

            unless file.empty? or file.nil?
                dir = file.sub(/[^\/]+$/,"")
                UI::EasinstallerApp.get_app.config["last_dir"] = dir

                session = Core::Session.new(file)

                notify(UI::Signals::NewSessionPossible)

                begin
                    rec_res = session.recognize_filetype
                rescue Core::Archive::InvalidArchive
                    AppToolkit::Message.new(EasinstallerApp.get_top_window, AppToolkit::Message::ERROR,
                        resources["corrupt1"] + file + resources["corrupt2"],
                        resources["corrupt_title"]).execute
                        return
                end

                if rec_res
                    Thread.new{
                        UI::error_wrapper{
                            EasinstallerApp.get_top_window.session = session
                            
                            if session.unpack

                                if session.configure_present?()
                                    EasinstallerApp.get_top_window.title = session.name
                                    session.run
                                elsif session.autogen_present?()
                                    AppToolkit::Message.new(EasinstallerApp.get_top_window, AppToolkit::Message::ERROR,
                                        resources["autogen1"],
                                        resources["autogen2"]).execute
                                    session.run_autogen
                                    if session.configure_present?()
                                        AppToolkit::Message.new(EasinstallerApp.get_top_window, AppToolkit::Message::ERROR,
                                            resources["autogen_success1"],
                                            resources["autogen_success2"]).execute
                                        EasinstallerApp.get_top_window.title = session.name
                                        session.run
                                    else
                                        AppToolkit::Message.new(EasinstallerApp.get_top_window, AppToolkit::Message::ERROR,
                                            resources["autogen_failure1"],
                                            resources["autogen_failure2"]).execute
                                        session.cleanup()
                                        EasinstallerApp.get_top_window.session = nil
                                    end
                                elsif session.makefile_present?()
                                    AppToolkit::Message.new(EasinstallerApp.get_top_window, AppToolkit::Message::ERROR,
                                        resources["only_makefile1"],
                                        resources["only_makefile2"]).execute
                                    notify(UI::Signals::ConfigureFinished)
                                else
                                    AppToolkit::Message.new(EasinstallerApp.get_top_window, AppToolkit::Message::ERROR,
                                        resources["unsupported_package1"],
                                        resources["unsupported_package2"]).execute
                                    session.cleanup()
                                    EasinstallerApp.get_top_window.session = nil
                                end
                            
                            end
                        }
                    }
                else
                    AppToolkit::Message.new(EasinstallerApp.get_top_window, AppToolkit::Message::ERROR,
                        resources["unknown_archive1"] + file + resources["unknown_archive2"],
                        resources["unknown_archive3"]).execute
                end

            end
        end

        def session_list_trigger
            current_session = EasinstallerApp.get_top_window.session.nil? ? nil :
                EasinstallerApp.get_top_window.session
            SessionListWindow.new(EasinstallerApp.get_top_window,
                current_session).execute
        end

        def settings_trigger
            SettingsWindow.new(EasinstallerApp.get_top_window,
                EasinstallerApp.get_top_window.session).execute
        end

        def program_settings_trigger
            ProgramSettingsWindow.new(EasinstallerApp.get_top_window).execute
        end


        def configure_trigger
            Thread.new{
                UI::error_wrapper{
                    notify(UI::Signals::RunConfigure)
                }
            }
        end

        def build_trigger
            Thread.new{
                UI::error_wrapper{
                    notify(UI::Signals::RunBuild)
                }
            }
        end

        def install_trigger
            current_installation_variant = nil
            unless request(UI::Signals::LocalInstallation)
                pd = PasswordDialog.new(EasinstallerApp.get_top_window)
                if pd.execute == 0
                    return
                else
                    current_installation_variant = pd.current_installation_variant
                end
            end

            Thread.new{
                UI::error_wrapper{
                    notify(UI::Signals::RunInstall, current_installation_variant)
                }
            }
        end

        def quit_trigger
            notify(UI::Signals::ShuttingDown)
            exit
        end

        def about_trigger
            UI::About.new(EasinstallerApp.get_top_window).execute
        end

        def debug
            log "debug"
        end
    end
end

