#*******************************************************************************#
#                                                                               #
#                            E A S I N S T A L L E R                            #
#                                                                               #
# A window showing command history                                              #
#*******************************************************************************#
# Copyright (C) 2004 by Yuri Leikind.                                           #
#*******************************************************************************#
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public                           #
# License as published by the Free Software Foundation                          #
#                                                                               #
# This program is distributed in the hope that it will be useful,               #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             #
# General Public License for more details.                                      #
#                                                                               #
# You should have received a copy of the GNU General Public                     #
# License along with this program; if not, write to the Free Software           #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    #
#*******************************************************************************#
# $Id: command_list.rb,v 1.16 2004/07/09 16:01:38 leikind Exp $                 #
#*******************************************************************************#



module UI

    class MyTableItem < Fox::FXTableItem
        attr_accessor :my_font

        def drawContent(table, dc, x, y, w, h)
            if my_font.nil?
                super(table, dc, x, y, w, h)
            else
                mydc = Fox::FXDCWindow.new(table)
                mydc.setTextFont(my_font) # !!!
                super(table, mydc, x, y, w, h)
            end
        end
    end

    class CommandList < Fox::FXTable
        include Fox

        if $FXRUBY_VERSION_1_2
        
            def initialize(parent, command_history)
                super(parent, nil, 0,
                        TABLE_COL_SIZABLE|TABLE_ROW_SIZABLE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
    
                self.setTableSize(command_history.length + 1, 1)
    
                cheader = self.columnHeader
                crow = self.rowHeader
                cheader.setItemText(0, AppToolkit::Resources.instance["command"])
                
                
                self.setColumnWidth(0, 3000)
    
                command_history.each_with_index do |item, i|
    
                    crow.setItemText(i, (i + 1).to_s)
    
                    fxitem = MyTableItem.new(item)
                    fxitem.my_font = EasinstallerApp.get_app.console_font
                    fxitem.justify = -1
                    self.setItem(i, 0, fxitem)
    
                    if i.remainder(2) == 0
                        self.setCellColor(0, 0, FXRGB(240, 240, 255))
                    else
                        self.setCellColor(1, 0, FXRGB(255, 240, 240))
                    end
                end
            end
        
        else
        
            def initialize(parent, command_history)
                super(parent, 5, 5, nil, 0,
                    TABLE_COL_SIZABLE|TABLE_ROW_SIZABLE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
    
                self.setTableSize(command_history.length + 1, 2)
    
                self.leadingRows = 1
                self.leadingCols = 1
    
                self.setColumnWidth(1, 3000)
                self.setColumnWidth(0, 40)
    
                fxitem = MyTableItem.new("N")
                fxitem.setButton(true)
                fxitem.my_font = EasinstallerApp.get_app.normalFont
                fxitem.justify = 0
                self.setItem(0, 0, fxitem)
    
                fxitem = MyTableItem.new(AppToolkit::Resources.instance["command"])
                fxitem.setButton(true)
                fxitem.my_font = EasinstallerApp.get_app.normalFont
                fxitem.justify = -1
                self.setItem(0, 1, fxitem)
    
                command_history.each_with_index do |item, i|
    
                    fxitem = MyTableItem.new((i+1).to_s)
                    fxitem.setButton(true)
                    fxitem.my_font = EasinstallerApp.get_app.normalFont
                    fxitem.justify = 0
                    self.setItem(i + 1, 0, fxitem)
    
                    fxitem = MyTableItem.new(item)
                    fxitem.my_font = EasinstallerApp.get_app.console_font
                    fxitem.justify = -1
                    self.setItem(i + 1, 1, fxitem)
    
                    if i.remainder(2) == 0
                        self.setCellColor(0, 1, FXRGB(240, 240, 255))
                    else
                        self.setCellColor(1, 1, FXRGB(255, 240, 240))
                    end
                end
            end
    
        end
    end
end
