#*******************************************************************************#
#                                                                               #
#                            E A S I N S T A L L E R                            #
#                                                                               #
# Core application UI                                                           #
#*******************************************************************************#
# Copyright (C) 2004 by Yuri Leikind.                                           #
#*******************************************************************************#
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public                           #
# License as published by the Free Software Foundation                          #
#                                                                               #
# This program is distributed in the hope that it will be useful,               #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             #
# General Public License for more details.                                      #
#                                                                               #
# You should have received a copy of the GNU General Public                     #
# License along with this program; if not, write to the Free Software           #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    #
#*******************************************************************************#
# $Id: easinstaller.rb,v 1.61 2004/07/09 16:01:38 leikind Exp $                 #
#*******************************************************************************#


module UI

    class EasinstallerApp < AppToolkit::Application

        include Fox

        def self.get_app
            @@app
        end

        def self.get_top_window
            @@window
        end

        attr_reader :console_font

        def initialize(cli_dir, cli_file)

            prerun_action = Proc.new{
                check = PreCheck.new(@resources)
                check.check_prerequisites()
            }
        
            super("Easinstaller", "resources/resources.", "resources/resource_table.yaml",
                "YuriLeikind", prerun_action, nil, ".easinstaller", "config.yaml")

            @console_font = FXFont.new(self, "courier", 10)

            @@app = self
            @window = TopWindow.new(self)
            @@window = @window

            if cli_dir != nil or cli_file != nil
                self.extend(MenuAndToolbarHooks)
                if cli_dir
                    open_directory_trigger(cli_dir)
                elsif cli_file
                    open_trigger(cli_file)
                end
            end
            run_
        end

    end

    class TopWindow < AppToolkit::MainWindow

        include Fox

        attr_reader :session

        def session=(session)
            @session = session
            notify(UI::Signals::NewSessionStarted, session)
        end

        def title=(title)
            super(@app.title + ": " + title)
        end

        def initialize(app)
            super(app, app.resources, app.icons, "ready")

            make_menu_and_toolbar()


            contents = FXSplitter.new(self,
                 (LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_VERTICAL|SPLITTER_TRACKING))

            coptions = COptions.new(contents)
            BottomTabBook.new(contents)

            self.connect(SEL_CLOSE){
                notify(UI::Signals::ShuttingDown)
                exit
            }

            set_hooks
            set_session_hooks

            @session = nil
            
        end


        def set_session_hooks
            hook(UI::Signals::RunConfigure){|emmitter|
                @session.safe_send(:run_configure)
            }

            hook(UI::Signals::RunBuild){|emmitter|
                @session.safe_send(:run_build)
            }

            hook(UI::Signals::RunInstall){|emmitter, installation_variant|
                @session.safe_send(:run_install, installation_variant)
            }

            hook(UI::Signals::LocalInstallation){|emmitter|
                @session.safe_send(:local_installation?)
            }

            hook(UI::Signals::ShuttingDown){|emmitter|
                @session.safe_send(:save)
            }

            hook(UI::Signals::NewSessionPossible){|emmitter|
                @session.safe_send(:save)
            }
        end

        def set_hooks
            hook(UI::Signals::ShuttingDown){|emmitter, filename|
                save_parameters
            }
            
            # Ctrl-C signal handler
            trap 2, Proc.new{
                log "see you"
                save_parameters
                notify(UI::Signals::ShuttingDown)
                exit(0)
            } 

            hook(UI::Signals::ConfigureFailed){
                AppToolkit::Message.new(EasinstallerApp.get_top_window,
                    AppToolkit::Message::ERROR, @app.resources["conf_fail_message"],
                    @app.resources["conf_fail_title"]).execute
            }

            hook(UI::Signals::NewCommand){|emmitter, command, comment|
                @statusbar.statusline.normalText = comment
            }

            hook(UI::Signals::CommandFinished){|emmitter|
                @statusbar.statusline.normalText = @app.resources["ready"]
            }
        end

        def save_parameters
            @app.config["width"] = self.width
            @app.config["height"] = self.height
            @app.config["x"] = self.x
            @app.config["y"] = self.y
            @app.config.dump
        end
    end


    class COptions < Fox::FXVerticalFrame
        include Fox
        def initialize(parent)
            super(parent, AppToolkit::FRAME_STYLE|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,
                (TopWindow::HEIGHT-70)/2, 0,0,0,0)
            set_hooks()
            @coptions = Array.new
        end

        def set_hooks()
            hook(UI::Signals::OptionsParsed){|emmitter, options|
                UI::error_wrapper{
                    build(options)
                }
            }

            hook(UI::Signals::NewSessionStarted){
                UI::error_wrapper{
                    clearup()
                }
            }

            hook(UI::Signals::LoadSettings){|emmitter, foreign_options|
                UI::error_wrapper{
                    load_settings(foreign_options)
                }
            }
        end

        def load_settings(foreign_options)
            @coptions.each{|coption|
                foreign_options.each{|opt|
                    coption.load_from(opt)
                }
            }
        end

        def clearup()
            if self.children[0].class == FXTabBook
                obj = self.children[0]
                self.removeChild(obj)
                @coptions.clear
            end
        end

        def build(options)
            option_pane = FXTabBook.new(self, nil, 0,  LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT)
            options.section_names.each do |sect|
               FXTabItem.new(option_pane, sect, nil)
               scroll_window = FXScrollWindow.new(option_pane, VSCROLLER_ALWAYS|LAYOUT_FILL_X|LAYOUT_FILL_Y)

               ver_cont = FXVerticalFrame.new(scroll_window, AppToolkit::FRAME_STYLE|LAYOUT_FILL_X|LAYOUT_FILL_Y)

               FXHorizontalSeparator.new(ver_cont, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|SEPARATOR_GROOVE)
               matrix = FXMatrix.new(ver_cont, 4, MATRIX_BY_COLUMNS|LAYOUT_FILL_X, DEFAULT_SPACING,
                    DEFAULT_SPACING, DEFAULT_SPACING, DEFAULT_SPACING, 30, 30)

               options.each_in_section(sect) do |o|
                    @coptions << COption.new(matrix, o)
               end
            end

            option_pane.create
            option_pane.recalc
        end
    end


    class COption
        include Fox

        def initialize(parent, option)
            @option = option
            @parent = parent
            @param  = (@option.class == Core::ParamOption) || (@option.class == Core::ParamBoolOption)
            build()
            set_actions()
        end

        def build
            @use = FXCheckButton.new(@parent, "", nil, 0, CHECKBUTTON_NORMAL|LAYOUT_SIDE_LEFT)

            @use.checkState = @option.on

            description = @option.descr.word_split(60).join("\n")
            FXLabel.new(@parent, description + "\t--" + @option.name, nil, LABEL_NORMAL|JUSTIFY_LEFT)

            if @param
                @textfield = FXTextField.new(@parent, 20, nil, 0,
                    FRAME_SUNKEN|LAYOUT_SIDE_TOP|LAYOUT_FILL_X)
                @textfield.disable unless @option.on

                @textfield.text = @option.param
            else
                FXLabel.new(@parent, "", nil, LABEL_NORMAL)
            end

            FXLabel.new(@parent, "", nil, LABEL_NORMAL)
        end

        def load_from(opt)
            if opt.name == @option.name and opt.class == @option.class
                log opt.name

                @option.on = opt.on
                @use.checkState = opt.on

                if @param
                    if opt.on
                        @textfield.enable
                    else
                        @textfield.disable
                    end

                    @textfield.text = opt.param
                    @option.param = opt.param
                end
            end
        end

        def set_actions
            if @param
                @textfield.connect(SEL_CHANGED){|sender, sel, data|
                    @option.param = data
                    log @option.to_s
                }
            end

            @use.connect(SEL_COMMAND){|sender, sel, data|
                if @param
                    if data
                        @textfield.enable
                    else
                        @textfield.disable
                    end
                end
                @option.on = data
                log @option.to_s
            }
        end
    end

    class BottomTabBook < Fox::FXTabBook
        include Fox
        def initialize(parent)
            super(parent, nil, 0,  LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT)
            resources = AppToolkit::Resources.instance
            tabitem = FXTabItem.new(self, resources["command_output"], nil)

            tabitem.easinstaller_text_key = {
                "text=" => "command_output"
            }

            Console.new(self)
            set_hooks()
            @deletable = Array.new
        end

        @@files = ["README", "INSTALL", "COPYING", "AUTHORS", "TODO",
            "NEWS", "ChangeLog", "CHANGELOG"]

        def make_tab(path, name)
            @deletable << FXTabItem.new(self, name, nil)

            textframe = FXHorizontalFrame.new(self,
                LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|AppToolkit::FRAME_STYLE)

            txt = FXText.new(textframe, nil, 0,
                LAYOUT_FILL_X|LAYOUT_FILL_Y|TEXT_READONLY|TEXT_WORDWRAP|VSCROLLER_ALWAYS)
            txt.setText(IO.readlines(path).join(""))
            #@deletable << txt
            @deletable << textframe
        end

        def clearup()
            @deletable.each{|obj|
                self.removeChild(obj)
            }
            @deletable.clear
        end

        def set_hooks
            hook(UI::Signals::NewSessionStarted){
                UI::error_wrapper{
                    clearup()
                }
            }

            hook(UI::Signals::OptionsParsed){
                session_path = UI::EasinstallerApp.get_top_window.session.src_dir
                @@files.each{|f|
                    path = session_path + "/" + f
                    if FileTest.exist?(path)
                        make_tab(path, f)
                    end
                }
                self.create
                self.recalc
            }
        end
    end

    class Console < Fox::FXVerticalFrame
        include Fox
        def initialize(parent)
            super(parent, AppToolkit::FRAME_STYLE|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0)

            top_container = FXHorizontalFrame.new(self, LAYOUT_SIDE_TOP|LAYOUT_FILL_X)
            @resources = AppToolkit::Resources.instance
            clabel = FXLabel.new(top_container, @resources["current_command"], nil,
                LABEL_NORMAL, 0, 0, 0, 0, DEFAULT_PAD, DEFAULT_PAD, 5, 5)
            clabel.tipText = @resources["command_explained"]
            clabel.helpText = @resources["command_explained"]

            clabel.easinstaller_text_key = {
                "text=" => "current_command",
                "tipText=" => "command_explained",
                "helpText=" => "command_explained"
            }

            @command_field = FXTextField.new(top_container, 50, nil, 0,
                TEXTFIELD_READONLY|FRAME_SUNKEN|LAYOUT_SIDE_TOP|LAYOUT_FILL_X)

            console_font = EasinstallerApp.get_app.console_font

            @command_field.font  = console_font
            @clear_butt = FXButton.new(top_container, @resources["clear"],
                    nil, nil, 0, AppToolkit::BUTTON_STYLE|ICON_AFTER_TEXT, 0,0,0,0,10, 10, 5, 5)
                    

            icons = EasinstallerApp.get_app.icons
            @clear_icon = icons.get_icon("icons/clear.png")
            @stop_icon = icons.get_icon("icons/stop.png")
            
            @clear_butt.setIcon(@clear_icon)      
            @clear_butt.tipText = @resources["clear_explained"]
            @clear_butt.helpText = @resources["clear_explained"]
            
            class << @clear_butt
                attr_accessor :state
            end
            
            @clear_butt.state = :clear

            @clear_butt.easinstaller_text_key = {
                "text=" => "clear",
                "helpText=" => "clear_explained",
                "tipText="  => "clear_explained"
            }

            textframe = FXHorizontalFrame.new(self,
                LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|AppToolkit::FRAME_STYLE)

            @text = FXText.new(textframe, nil, 0,
                LAYOUT_FILL_X|LAYOUT_FILL_Y|TEXT_READONLY|TEXT_WORDWRAP|VSCROLLER_ALWAYS)

            @text.font = console_font

            @text.styled = true

            hs1 = FXHiliteStyle.new
            hs1.normalForeColor = FXRGB(255, 0, 0)
            hs1.style = 0

            hs2 = FXHiliteStyle.new
            hs2.normalForeColor = FXRGB(0, 100, 0)
            hs2.style = 0

            @text.hiliteStyles = [hs1, hs2]

            @clear_butt.connect(SEL_COMMAND) do
                if @clear_butt.state == :clear
                    @text.setText("")
                elsif @clear_butt.state == :stop
                    @ipc.stop
                else
                    raise "invalid state of @clear_butt button: #{@clear_butt.state}"
                end
            end
            set_hooks
            
            @ipc = Core::IPC.instance
            @ipc.register_direct_output_listener(self)            
        end


        def stderr_listen(string)
            @text.appendStyledText(string, 1)
            @text.makePositionVisible(@text.length)        
        end

        def stdout_listen(string)
            @text.appendStyledText(string, 2)
            @text.makePositionVisible(@text.length)        
        end

        
        def set_hooks
=begin        
            hook(UI::Signals::ConsoleLineStdErr){|emmitter, line|
                @text.appendStyledText(line, 1)
                @text.makePositionVisible(@text.length)
            }

            hook(UI::Signals::ConsoleLineStdOut){|emmitter, line|
                @text.appendStyledText(line, 2)
                @text.makePositionVisible(@text.length)
            }
=end
    

            hook(UI::Signals::NewCommand){|emmitter, command, comment|
                @command_field.text = command
                
                @clear_butt.text     = @resources["stop"]
                @clear_butt.tipText  = @resources["stop_explained"]
                @clear_butt.helpText = @resources["stop_explained"]

                @clear_butt.easinstaller_text_key = {
                    "text=" => "stop",
                    "helpText=" => "stop_explained",
                    "tipText="  => "stop_explained"
                }
                @clear_butt.state = :stop
                @stop_icon.create
                @clear_butt.setIcon(@stop_icon)
            }

            hook(UI::Signals::CommandFinished){|emmitter, command, comment|
                @clear_butt.text     =  @resources["clear"]
                @clear_butt.tipText  =  @resources["clear_explained"]
                @clear_butt.helpText =  @resources["clear_explained"]

                @clear_butt.easinstaller_text_key = {
                    "text=" => "clear",
                    "helpText=" => "clear_explained",
                    "tipText="  => "clear_explained"
                }
                @clear_butt.state = :clear
                @clear_icon.create
                @clear_butt.setIcon(@clear_icon)
            }
            
            
            hook(UI::Signals::NewSessionStarted){
                @text.setText("")
            }
        end
    end

    def self.error_wrapper()
        begin
            yield
        rescue Exception
            STDERR.puts $!
            STDERR.puts $!.backtrace

            AppToolkit::Message.new(EasinstallerApp.get_top_window,
                AppToolkit::Message::ERROR, $!.to_s + "\n\n" + $!.backtrace.join("\n"),
                @resources["program_error"]).execute
        end
    end
end
