#*******************************************************************************#
#                                                                               #
#                            E A S I N S T A L L E R                            #
#                                                                               #
# Password dialog                                                               #
#*******************************************************************************#
# Copyright (C) 2004 by Yuri Leikind.                                           #
#*******************************************************************************#
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public                           #
# License as published by the Free Software Foundation                          #
#                                                                               #
# This program is distributed in the hope that it will be useful,               #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             #
# General Public License for more details.                                      #
#                                                                               #
# You should have received a copy of the GNU General Public                     #
# License along with this program; if not, write to the Free Software           #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    #
#*******************************************************************************#
# $Id: password_dialog.rb,v 1.19 2004/07/22 12:46:15 leikind Exp $               #
#*******************************************************************************#

module UI

    class BaseInstallationVariant  < Fox::FXVerticalFrame
        include Fox
        
        def initialize(parent, dialog)
            super(parent, LAYOUT_FILL_X|LAYOUT_FILL_Y)
            @dialog = dialog
            @resources = AppToolkit::Resources.instance
        end
    end

    class InstallationWithPasswordVariant  < BaseInstallationVariant
        include Fox
        def password
            @passwd.text
        end
        
        def create_passwd_field(parent)
            tf = FXTextField.new(parent, 12, nil, 0,
                TEXTFIELD_PASSWD|AppToolkit::FRAME_STYLE|LAYOUT_FILL_X|LAYOUT_CENTER_Y)
                
            tf.connect(SEL_KEYRELEASE){|sender, sel, event|
                if event.code == 65293  # Enter
                    @dialog.ok()
                end
            }
            
            return tf
        end
        
        def password_valid?
            if password().strip == ""
                [false, @resources["missing_password"], @resources["password_empty"]]
            else
                [true]
            end
        end
    end

    
    class RootInstallation < InstallationWithPasswordVariant
        include Fox
        def initialize(parent, dialog)
            super(parent, dialog)
            
            password_frame = FXHorizontalFrame.new(self, LAYOUT_FILL_X|LAYOUT_FILL_Y)
            FXLabel.new(password_frame, @resources["root_password"], nil, LAYOUT_SIDE_LEFT|LAYOUT_CENTER_Y)
            
            @passwd = create_passwd_field(password_frame)
        end

        def password_valid?
            blank_password_check = super()
            if blank_password_check[0] == false
                return blank_password_check
            end
 
            if request(UI::Signals::ValidPassword, password())
                [true]
            else
                [false, @resources["invalid_root_password"], 
                    @resources["invalid_root_password"]]
            end
        end
    end

    class UserInstallation < BaseInstallationVariant
        include Fox
        def initialize(parent, dialog)
            super(parent, dialog)
            #user_frame = FXHorizontalFrame.new(@switcher, LAYOUT_FILL_X|LAYOUT_FILL_Y)
            FXLabel.new(self, @resources["installing_as_user"], nil, 
                LAYOUT_SIDE_LEFT|LAYOUT_CENTER_Y|JUSTIFY_LEFT)
        end
    end
    
    class DisabledInstallation < BaseInstallationVariant
        include Fox
        def initialize(parent, label_message)
            super(parent, nil)

            FXLabel.new(self, label_message, nil, 
                LAYOUT_SIDE_LEFT|LAYOUT_CENTER_Y|JUSTIFY_LEFT)
        end    
    end
    
    
    class SudoInstallationDisabled < DisabledInstallation
        def initialize(parent, dialog)
            super(parent, @resources["installing_via_sudo1"])
        end
    end


    class RootInstallationDisabled < DisabledInstallation
        def initialize(parent, dialog)
            super(parent, @resources["installing_as_root1"])
        end
    end

    
    class SudoInstallation < InstallationWithPasswordVariant
        include Fox
        def initialize(parent, dialog)
            super(parent, dialog)
            

            FXLabel.new(self, @resources["installing_via_sudo4"], nil, 
                LAYOUT_SIDE_LEFT|LAYOUT_CENTER_Y|JUSTIFY_LEFT)
            
            @cb = FXCheckButton.new(self, @resources["installing_via_sudo5"], nil, 0, 
                CHECKBUTTON_NORMAL|LAYOUT_SIDE_LEFT)
                
            @cb.connect(SEL_COMMAND){|sender, sel, data|
                if data
                    @passwd.enable
                else
                    @passwd.disable
                end
            }
                
            sudo_frame = FXHorizontalFrame.new(self, LAYOUT_FILL_X|LAYOUT_FILL_Y)
            FXLabel.new(sudo_frame, @resources["user_password"], nil, LAYOUT_SIDE_LEFT|LAYOUT_CENTER_Y)
            
            @passwd = create_passwd_field(sudo_frame)
                
            if request(UI::Signals::PasswordRequiredForSudo)
                @cb.checkState = true
                @passwd.enable
            else
                @cb.checkState = false
                @passwd.disable    
            end
        end
        
        def using_password?
            @cb.checkState == true or @cb.checkState == 1 
        end
        
        def password_valid?
        
            if using_password?
                blank_password_check = super()
                if blank_password_check[0] == false
                    return blank_password_check
                end
                if request(UI::Signals::IsPasswordValidForSudo, password())
                    if request(UI::Signals::SudoAllowsCommand, password())
                        [true]
                    else
                        [false, @resources["sudo_error"], @resources["sudo_error_expl_1"] +
                            ENV["USER"] + "       ALL = NOPASSWD: /usr/bin/make install" ]
                    end
                else
                    [false, @resources["invalid_user_password"], @resources["invalid_user_password2"]]
                end
            else
                if request(UI::Signals::PasswordRequiredForSudo)
                    @cb.checkState = true
                    @passwd.enable
                    [false, @resources["password_required"], @resources["password_required2"]]
                else
                    [true]
                end
            end
        end         
    end
    
    
    class PasswordDialog < AppToolkit::Dialog
        include Fox

        def initialize(parent)
        
            actions = {
                AppToolkit::Dialog::OK => Proc.new{},
                AppToolkit::Dialog::CANCEL => Proc.new{}
            }
            
            super(parent, "installing_package", 500, 300, actions)        
        
            option_frame = FXVerticalFrame.new(@content, AppToolkit::FRAME_STYLE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
            
            FXLabel.new(option_frame, @resources["installation_variants"], nil, 
                JUSTIFY_HZ_APART|LAYOUT_SIDE_LEFT|LAYOUT_CENTER_Y)
            
            # Before the 1.2.x FoX and FXRuby series FXGroupBox could be used to 
            # group radiobuttons. The new versions do not provide this magic, so,
            # using FXDataTarget is reasonable, but radiobuttons with FXDataTarget
            # work well only if threadsEnabled = false, which is inacceptable.
            # So, the best way here is to ensure mutually exclusive behavior for
            # radiobuttons manually, like this:
            #         rb2.checkState = false
            #         rb3.checkState = false
                
            #gp = FXGroupBox.new(option_frame, "", LAYOUT_SIDE_TOP|LAYOUT_FILL_X, 0, 0, 0, 0)
            gp = FXVerticalFrame.new(option_frame, FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
            
            rb1 = FXRadioButton.new(gp, @resources["install_as_root"], nil, 0, 
                ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP)
                
            rb2 = FXRadioButton.new(gp, @resources["install_as_user"], nil, 0, 
                ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP)
                
            rb3 = FXRadioButton.new(gp, @resources["install_via_sudo"], nil, 0, 
                ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP)
                
            rb1.connect(SEL_COMMAND){
                if CAPABILITIES["expect"]
                    @buttons["ok"].enable
                else
                    @buttons["ok"].disable
                end
                @switcher.setCurrent(0)
                rb2.checkState = false
                rb3.checkState = false
            }
            
            rb2.connect(SEL_COMMAND){
                @buttons["ok"].enable
                @switcher.setCurrent(1)
                rb1.checkState = false
                rb3.checkState = false                    
            }
            
                
            rb3.connect(SEL_COMMAND){
                if CAPABILITIES["sudo"]
                    @buttons["ok"].enable
                else
                    @buttons["ok"].disable
                end
                @switcher.setCurrent(2)
                rb1.checkState = false
                rb2.checkState = false                    
            }

            @switcher = FXSwitcher.new(option_frame,
                LAYOUT_FILL_X|LAYOUT_FILL_Y)

            sudo_installation_class = CAPABILITIES["sudo"] ? SudoInstallation : SudoInstallationDisabled
            root_installation_class = CAPABILITIES["expect"] ? RootInstallation : RootInstallationDisabled
            
            @installation_variants = [root_installation_class.new(@switcher, self),
                UserInstallation.new(@switcher, self), sudo_installation_class.new(@switcher, self)]
            
            if CAPABILITIES["expect"]
                @buttons["ok"].enable
            else
                @buttons["ok"].disable
            end
            
            @config = UI::EasinstallerApp.get_app.config
            inst_var = @config["last_installation_variant"]
            
            if inst_var.class == String and inst_var =~ /^[123]$/
                eval("rb" + inst_var + ".checkState = true")
                @switcher.setCurrent(inst_var.to_i - 1)
            else
                rb1.checkState = true
            end            
        end

        def current_installation_variant
            @installation_variants[@switcher.current]
        end
        
        def remember_installation_variant
            @config["last_installation_variant"] = (@switcher.current + 1).to_s
        end
        
        def validate
            current_variant = current_installation_variant()
            if current_variant.respond_to?(:password_valid?)
                validation_data = current_variant.password_valid?
                if validation_data[0]
                    remember_installation_variant
                    true
                else
                    AppToolkit::Message.new(EasinstallerApp.get_top_window, AppToolkit::Message::ERROR,
                        validation_data[2], validation_data[1]).execute
                    false
                end
            else
                remember_installation_variant
                true
            end
        end 
    end
end    
