#*******************************************************************************#
#                                                                               #
#                            E A S I N S T A L L E R                            #
#                                                                               #
# Program settings dialog                                                       #
#*******************************************************************************#
# Copyright (C) 2004 by Yuri Leikind.                                           #
#*******************************************************************************#
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public                           #
# License as published by the Free Software Foundation                          #
#                                                                               #
# This program is distributed in the hope that it will be useful,               #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             #
# General Public License for more details.                                      #
#                                                                               #
# You should have received a copy of the GNU General Public                     #
# License along with this program; if not, write to the Free Software           #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    #
#*******************************************************************************#
# $Id: program_settings.rb,v 1.34 2004/07/14 14:37:00 leikind Exp $             #
#*******************************************************************************#


module UI
    # some code from shutter.rb in FXRuby examples:
    class ShutterItem < Fox::FXShutterItem
        include Fox
        def initialize(parent, text, icon=nil, opts=0)
            super(parent, text, icon,
                opts|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT, 0, 0, 0, 0, 10, 10, 10,
                10, 10, 10)
            button.padTop = 2
            button.padBottom = 2
        end
    end

    class ShutterButton < Fox::FXButton
        include Fox
        def initialize(parent, text, icon=nil)
            super(parent, text, icon, nil, 0,
                BUTTON_TOOLBAR|TEXT_BELOW_ICON|FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|
                LAYOUT_TOP|LAYOUT_LEFT)
            self.backColor = parent.backColor
            self.textColor = FXRGB(255, 255, 255)
        end
    end


    class ProgramSettingsWindow < AppToolkit::Dialog

        include Fox

        def initialize(parent)
        
            actions = {
                AppToolkit::Dialog::OK => Proc.new{
                    apply()
                },
                AppToolkit::Dialog::APPLY => Proc.new{
                    apply()
                },
                AppToolkit::Dialog::CANCEL => Proc.new{}
            }
        
            super(parent, "settings", 700, 290, actions)
        
            splitter = FXSplitter.new(@content, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|
                LAYOUT_FILL_Y|SPLITTER_TRACKING)


            @shutter = FXShutter.new(splitter, nil, 0,
                FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0, 0, 200)


            shutterItem = ShutterItem.new(@shutter, @resources["common_settings"],
                nil, LAYOUT_FILL_Y)

            shutterItem.button.easinstaller_text_key = { "text=" => "common_settings" }

            icons_loader = EasinstallerApp.get_app.icons
            working_directory_icon = icons_loader.get_icon("icons/working_directory.png")

            b1 = ShutterButton.new(shutterItem.content, @resources["working_directory2"],
                working_directory_icon)

            b1.easinstaller_text_key = { "text=" => "working_directory2" }

            b1.connect(SEL_COMMAND){
                @switcher.current = 0
            }

            shutterItem = ShutterItem.new(@shutter, @resources["appearance"])

            shutterItem.button.easinstaller_text_key = { "text=" => "appearance" }


            lang_icon = icons_loader.get_icon("icons/world.png")

            b2 = ShutterButton.new(shutterItem.content, @resources["lang"],
                lang_icon)
            b2.easinstaller_text_key = { "text=" => "lang" }
            b2.connect(SEL_COMMAND){
                    @switcher.current = 1
            }


            font_icon = icons_loader.get_icon("icons/fonts.png")

            b3 = ShutterButton.new(shutterItem.content, @resources["fonts"],
                font_icon)
            b3.easinstaller_text_key = { "text=" => "fonts" }
            b3.connect(SEL_COMMAND){
                    @switcher.current = 2
            }

            @switcher = FXSwitcher.new(splitter,
                FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0, 0, 0, 0, 10, 10, 10, 10)

            @working_dir_settings = WorkingDirectorySettings.new(@switcher)
            @lang_settings = LanguageSettings.new(@switcher)
            @font_settings = FontSettings.new(@switcher)
        end

        def apply()
            @working_dir_settings.commit() if @working_dir_settings.changed

            if @lang_settings.changed
                @lang_settings.commit()
                @font_settings.update_button(EasinstallerApp.get_app.normalFont, false)
            end
            @font_settings.commit() if @font_settings.changed
        end
    end


    class SomeSetting
        include Fox

        attr_reader :changed

        def commit
            @changed = false
        end

        def initialize(parent, label)
            @changed = false
            @frame = FXGroupBox.new(parent, AppToolkit::Resources.instance[label],
                GROUPBOX_TITLE_CENTER|FRAME_RIDGE|LAYOUT_FILL_X|LAYOUT_FILL_Y,
                0, 0, 0, 0, 10, 10, 10, 10)
            @frame.easinstaller_text_key = { "text=" => label }
        end
    end

    class WorkingDirectorySettings  < SomeSetting

        def path
            @path_entry.text
        end

        def initialize(parent)
            super(parent, "working_directory2")
            
            resources = AppToolkit::Resources.instance
            
            FXLabel.new(@frame, resources["working_directory_explanation"],
                nil, JUSTIFY_LEFT).easinstaller_text_key = {
                    "text=" => "working_directory_explanation"
                }

            horizontal = FXHorizontalFrame.new(@frame,
                LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y)
            @path_entry = FXTextField.new(horizontal, 40, nil, 0,
                    FRAME_SUNKEN|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|TEXTFIELD_READONLY)
            @path_entry.text = Core::SrcDirectory.instance.global_src.to_s

            change_path_butt = FXButton.new(horizontal, resources["change"],
                nil, nil, 0, AppToolkit::BUTTON_STYLE)

            change_path_butt.easinstaller_text_key = { "text=" => "change" }

            change_path_butt.connect(SEL_COMMAND) do
                dir = FXFileDialog.getOpenDirectory(EasinstallerApp.get_top_window,
                    resources["new_working_directory"], @path_entry.text)
                unless dir.nil? or dir == ""
                    if @path_entry.text != dir
                        @path_entry.text = dir
                        @changed = true
                    else
                        @changed = false
                    end
                end
            end
        end

        def commit
            log "commiting dirs"
            Core::SrcDirectory.instance.global_src = self.path()
            super()
        end

    end

    class LanguageSettings  < SomeSetting
        def initialize(parent)
            super(parent, "lang")
            
            resources = AppToolkit::Resources.instance

            FXLabel.new(@frame, resources["languages_explanation"],
                nil, JUSTIFY_LEFT).easinstaller_text_key = {
                    "text=" => "languages_explanation"
                }

            horizontal = FXHorizontalFrame.new(@frame,
                LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y)

            @locales = EasinstallerApp.get_app.locale
            list = @locales.lang_list

            combobox = FXComboBox.new(horizontal, list.length, 0, nil, 0,
                AppToolkit::FRAME_STYLE|COMBOBOX_STATIC|LAYOUT_FILL_X)
                
            combobox.numVisible = (list.length < 5) ? list.length : 5

            current = @locales.current_locale
            refresh_table = Array.new
            list.each_with_index {|tuple, i|
                key, name = tuple
                combobox.appendItem(resources[name])
                refresh_table[i] = name
                if key == current
                    combobox.currentItem = i
                    @initial_lang = key
                end
            }

            combobox.easinstaller_text_key = { "setItemText" => refresh_table }

            combobox.connect(SEL_COMMAND) {
                @choosen_lang = list[combobox.currentItem][0]

                if @initial_lang == @choosen_lang
                    @changed = false
                else
                    @changed = true
                end
            }

        end

        def commit
            log "commiting langs"
            @locales.set_locale(@choosen_lang)
            @initial_lang = @choosen_lang

            new_font = @locales.get_font(EasinstallerApp.get_app)
            EasinstallerApp.get_app.normalFont = new_font

            Fox.update_text()
            Fox.update_font(new_font)

            super()
        end
    end


    class FontSettings  < SomeSetting
        def initialize(parent)
            super(parent, "fonts")

            @resources = AppToolkit::Resources.instance
            @locale  = UI::EasinstallerApp.get_app.locale
            
            FXLabel.new(@frame, @resources["font_explanation"],
                nil, JUSTIFY_LEFT).easinstaller_text_key = {
                    "text=" => "font_explanation"
                }

            cb = FXCheckButton.new(@frame, @resources["predefined"], nil, 0, 
                CHECKBUTTON_NORMAL)

            cb.easinstaller_text_key = { "text=" => "predefined" }

            horizontal = FXHorizontalFrame.new(@frame,
                LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y)

            initial_font = UI::EasinstallerApp.get_app.normalFont

            @butt = FXButton.new(horizontal, font_description(initial_font), nil,
                nil, 0, AppToolkit::BUTTON_STYLE|LAYOUT_FILL_X)

            @predefined = EasinstallerApp.get_app.locale.using_predefined_font?

            if @predefined
                @butt.disable
                cb.checkState = true
            end

            cb.connect(SEL_COMMAND){|sender, sel, data|
                if data
                    @butt.disable
                    synchronize_font()
                    @predefined = true
                    update_button(@new_font)
                else
                    @predefined = false
                    @butt.enable
                end
            }

            @butt.connect(SEL_COMMAND){
                dialog = FXFontDialog.new(@frame, "name")
                if dialog.execute == 1
                    @new_font = FXFont.new(EasinstallerApp.get_app, dialog.fontSelection)
                    @new_font.create()
                    update_button(@new_font)
                end
            }

        end

        def update_button(font, change = true)
            @butt.text = font_description(font)
            @butt.font = font
            @changed = true if change
        end

        def font_description(font)
            fontd = font.fontDesc
            "\n  " + fontd.face.capitalize + " " + fontd.size.to_s + ":  " +
                @resources["fontcheck"] + "  \n"
        end
        
        def synchronize_font()
            @new_font = @locale.get_default_font(UI::EasinstallerApp.get_app)
        end

        def commit
            log "commiting font"
            synchronize_font()
            EasinstallerApp.get_app.normalFont = @new_font
            Fox.update_font(@new_font)

            if @predefined
                @locale.set_predefined()
            else
                @locale.save_font(@new_font)
            end
            super()
        end
    end
end
