#*******************************************************************************#
#                                                                               #
#                            E A S I N S T A L L E R                            #
#                                                                               #
# A window showing all sessions                                                 #
#*******************************************************************************#
# Copyright (C) 2004 by Yuri Leikind.                                           #
#*******************************************************************************#
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public                           #
# License as published by the Free Software Foundation                          #
#                                                                               #
# This program is distributed in the hope that it will be useful,               #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             #
# General Public License for more details.                                      #
#                                                                               #
# You should have received a copy of the GNU General Public                     #
# License along with this program; if not, write to the Free Software           #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    #
#*******************************************************************************#
# $Id: session_list.rb,v 1.48 2004/07/09 16:01:38 leikind Exp $                 #
#*******************************************************************************#


module UI
    require "fileutils.rb"

    class SessionListWindow < AppToolkit::Dialog

        include Fox

        def initialize(parent, current_session)
            actions = {
                AppToolkit::Dialog::OK => Proc.new{}
            }
            super(parent, "projects", 850, 350, actions, true)

            session_list = SessionList.new(@content, self, current_session)
        end
    end

    class SettingsWindow < AppToolkit::Dialog

        include Fox

        def initialize(parent, session)
            actions = {
                AppToolkit::Dialog::OK => Proc.new{}
            }
            super(parent, "conf", 700, 280, actions)
            SessionDescription.new(@content, session, self)
        end

        def change_name(session, new_name)
            self.title = new_name + @resources["conf"]
        end
    end


    class SessionList
        include Fox

        attr_reader :session_selected

        def initialize(parent, window_parent, current_session)
            @parent = parent

            list_wrapper= FXGroupBox.new(parent,"" ,
                FRAME_SUNKEN|LAYOUT_SIDE_TOP|LAYOUT_FILL_Y|LAYOUT_FIX_WIDTH, 0, 0, 162, 0)

            @list  = FXList.new(list_wrapper, 0, nil, 1,
                LAYOUT_FILL_Y|LAYOUT_FIX_WIDTH|LIST_SINGLESELECT,
                0, 0, 150, 0)            

            vertical_inner = FXVerticalFrame.new(parent,
                FRAME_NONE|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y)

            @switcher = FXSwitcher.new(vertical_inner,
                LAYOUT_FILL_X|LAYOUT_FILL_Y|SWITCHER_HCOLLAPSE|SWITCHER_VCOLLAPSE)

            @control_buttons = SessionControlButtons.new(vertical_inner, window_parent,
                self, current_session)

            @offset_table = Hash.new
            @session_list_table = Hash.new
            n = 0

            Core::SessionList.instance.get.each do |path, name|
                if current_session and path == current_session.src_dir
                    log "ommiting current session path #{path}"
                else
                    @session_list_table[n] = Core::ActionlessSession.new_from_directory(path)
                    @offset_table[n] = n
                    @list.appendItem(@session_list_table[n].name)
                    SessionDescription.new(@switcher, @session_list_table[n], self)
                    n += 1
                end
            end

            @control_buttons.enable = false if n == 0

            @session_selected = @session_list_table[0]

            @list.connect(SEL_SELECTED){|sender, sel, which|
                #puts "which=" + which.to_s
                #puts "session " + @session_list_table[which].name
                @session_selected = @session_list_table[which]
                @switcher.setCurrent(@offset_table[which])
                @control_buttons.enable = true
            }
        end

        def load_settings
            notify(UI::Signals::LoadSettings, @session_selected.options)
        end

        def remove(session)
            i = find_index_by_session(session)
            @list.removeItem(i)
            #@list.disableItem(i)
            @list.killSelection

            @control_buttons.enable = false

            @switcher.children[@offset_table[i]].destroy
            #puts "destroying #{@offset_table[i]}"
            @session_list_table[i] = nil

            recalc_table(i)
        end


        def recalc_table(i)
            loop do
                if @session_list_table.has_key?(i+1)
                    @session_list_table[i] = @session_list_table[i+1]
                    @offset_table[i] = @offset_table[i+1]
                    i += 1
                else
                    @session_list_table.delete(i)
                    break
                end
            end
        end

        def dump
            n = 0
            loop do
                if @session_list_table.has_key?(n)
                    log(n.to_s + " => ")
                    if @session_list_table[n].nil?
                        log("")
                    else
                        log(@session_list_table[n].name  + " => " + @offset_table[n].to_s)
                    end
                    n += 1
                else
                    break
                end
            end
        end

        def change_name(session, new_name)
            i = find_index_by_session(session)
            @list.setItemText(i, new_name)
        end

        private

        def find_index_by_session(session)
            @session_list_table.find{|k, v| v == session}[0]
        end
    end


    class SessionDescription
        include Fox

        def initialize(parent, session, session_list)
        
            resources = AppToolkit::Resources.instance
        
            switched_frame = FXVerticalFrame.new(parent,
                LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y)

            matrix = FXMatrix.new(switched_frame, 3, MATRIX_BY_COLUMNS|LAYOUT_FILL_X, DEFAULT_SPACING,
                DEFAULT_SPACING, DEFAULT_SPACING, DEFAULT_SPACING)

            FXLabel.new(matrix, resources["name"], nil, LABEL_NORMAL)
            name_entry = FXTextField.new(matrix, 30, nil, 0, FRAME_SUNKEN|LAYOUT_LEFT)

            name_entry.text = session.name

            name_entry.connect(SEL_CHANGED){|sender, sel, new_name|
                session.name = new_name
                session.save
                session_list.change_name(session, new_name) unless session_list.nil?
            }

            FXLabel.new(matrix, "", nil, LABEL_NORMAL)

            FXLabel.new(matrix, resources["working_directory"], nil, LABEL_NORMAL)

            path_entry = FXTextField.new(matrix, 30, nil, 0,
                FRAME_SUNKEN|LAYOUT_LEFT|TEXTFIELD_READONLY)

            path_entry.text = session.src_dir

            change_path_butt = FXButton.new(matrix, resources["move"], nil, nil, 0,
                AppToolkit::BUTTON_STYLE)

            change_path_butt.connect(SEL_COMMAND) do
                dir = Fox::FXFileDialog.getOpenDirectory(EasinstallerApp.get_top_window,
                    resources["new_working_directory"], session.src_dir)
                unless dir.nil? or dir == ""
                    path_entry.text = dir

                    FileUtils.mv(Dir.glob(session.src_dir+"/*").
                        reject{|e|e=='.' or 'e'=='..'}, dir)

                    # deleting parent directories which had been created automatically
                    parent = Pathname.new(session.src_dir).parent
                    FileUtils.rm_rf(session.src_dir)
                    if parent.parent ==  Core::SrcDirectory.instance.global_src and parent.basename.to_s =~ /^\d+$/
                        begin
                            parent.delete
                        rescue Errno::ENOTEMPTY
                            log("Failed to delete #{parent} - it is not empty")
                        end
                    end

                    session_list = Core::SessionList.instance 
                    # deleting from Core session list
                    session_list.delete(session.src_dir)

                    session.src_dir = dir
                    session.save

                    # adding to  Core session list anew
                    session_list.add(session)
                end
            end
            #path_entry.setFont(EasinstallerApp.get_app.console_font)

            FXLabel.new(matrix, resources["first_opened"], nil, LABEL_NORMAL)
            FXLabel.new(matrix, session.first_opened.to_s, nil, LABEL_NORMAL)
            FXLabel.new(matrix, "", nil, LABEL_NORMAL)

            FXLabel.new(matrix, resources["last_opened"], nil, LABEL_NORMAL)
            FXLabel.new(matrix, session.last_opened.to_s, nil, LABEL_NORMAL)

            UI::CommandList.new(switched_frame, session.command_history)
        end
    end


    class SessionControlButtons
        include Fox
        include UI::MenuAndToolbarHooks

        def initialize(parent, window_parent, session_list, current_session)
            resources = AppToolkit::Resources.instance
        
            horizontal = FXHorizontalFrame.new(parent,
                FRAME_NONE|LAYOUT_SIDE_TOP|LAYOUT_FILL_X)

            @open_button = FXButton.new(horizontal, resources["load"], nil, nil, 0,
                LAYOUT_CENTER_Y|LAYOUT_FILL_X|AppToolkit::BUTTON_STYLE)

            @open_button.tipText = resources["open_button_explained"]

            @open_button.connect(SEL_COMMAND) do

                open_directory_trigger(session_list.session_selected.src_dir)

                # This is how FoX messages are sent ;-))
                window_parent.handle(@open_button, MKUINT(FXDialogBox::ID_ACCEPT, SEL_COMMAND), nil )
            end

            if current_session
                @load_settings_button = FXButton.new(horizontal, resources["load_settings"],
                    nil, nil, 0,  LAYOUT_CENTER_Y|LAYOUT_FILL_X|AppToolkit::BUTTON_STYLE)

                @load_settings_button.tipText = resources["load_settings_explained"]

                @load_settings_button.connect(SEL_COMMAND) do
                    session_list.load_settings
                    window_parent.handle(@open_button, MKUINT(FXDialogBox::ID_ACCEPT, SEL_COMMAND), nil)
                end
            end

            @del_button = FXButton.new(horizontal, resources["delete"], nil, nil, 0,
                LAYOUT_CENTER_Y|LAYOUT_FILL_X|AppToolkit::BUTTON_STYLE)

            @del_button.tipText = resources["del_button_explained"]

            @del_button.connect(SEL_COMMAND) do

                session = session_list.session_selected

                # deleting from the filesystem
                session.cleanup()

                # deleting from FoX session list
                session_list.remove(session)

                # deleting from Core session list
                Core::SessionList.instance.delete(session.src_dir)
            end
        end

        def enable=(flag)
            @del_button.enabled = flag
            @open_button.enabled = flag
        end
    end
end
