#*******************************************************************************#
#                                                                               #
#                            E A S I N S T A L L E R                            #
#                                                                               #
# Miscellaneous tools                                                           #
#*******************************************************************************#
# Copyright (C) 2004 by Yuri Leikind.                                           #
#*******************************************************************************#
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public                           #
# License as published by the Free Software Foundation                          #
#                                                                               #
# This program is distributed in the hope that it will be useful,               #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             #
# General Public License for more details.                                      #
#                                                                               #
# You should have received a copy of the GNU General Public                     #
# License along with this program; if not, write to the Free Software           #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    #
#*******************************************************************************#
# $Id: tools.rb,v 1.12 2004/07/06 15:56:05 leikind Exp $                         #
#*******************************************************************************#

module UI

    CAPABILITIES = Hash.new

    class PreCheck
    
        def initialize(resources)
            @resources = resources
            @prerequisites = [
                ["make", @resources["make"], lambda{exit 20},
                 AppToolkit::Message::ERROR],
        
                ["expect", @resources["expect"], lambda{},
                 AppToolkit::Message::WARNING],
        
                ["unzip", @resources["unzip"], lambda{},
                 AppToolkit::Message::WARNING],
        
                ["gunzip", @resources["gunzip"], lambda{},
                 AppToolkit::Message::WARNING],
        
                ["bunzip2", @resources["bunzip2"], lambda{},
                 AppToolkit::Message::WARNING],
                 
                ["sudo", @resources["sudo_missing"], lambda{},
                 AppToolkit::Message::INFORMATION]         
             ]
             
             @config = UI::EasinstallerApp.get_app.config
        end

        def check_prerequisites
            @prerequisites.each {|tool, message, action, icon_type|
                CAPABILITIES[tool] = true
    
                if system("which #{tool} > /dev/null")
                    if @config["do_not_remind_about_" + tool] == true
                        @config["do_not_remind_about_" + tool] = false
                    end                
                else
                    if @config["do_not_remind_about_" + tool] != true
                        ToolWarningDialog.new(UI::EasinstallerApp.get_top_window, 
                            tool, message, @config).execute
                    end
                    action.call
                    CAPABILITIES[tool] = false
                end
            }
        end
    end
    
    class ToolWarningDialog < AppToolkit::Dialog
        include Fox

        def initialize(parent, tool, message, config)
                
            actions = {
                AppToolkit::Dialog::OK => Proc.new{process_reminding()}
            }
            
            @config, @tool = config, tool
            
            super(parent, tool + AppToolkit::Resources.instance["is_missing"], 
                0, 0, actions)
                
            FXLabel.new(@content,message + "\n",nil,
                JUSTIFY_LEFT|LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y)
            
            @use = FXCheckButton.new(@content, @resources["dont_remind"], nil, 0, 
                CHECKBUTTON_NORMAL|LAYOUT_SIDE_LEFT)

            @use.checkState = false   
            
            @use.connect(SEL_COMMAND){|sender, sel, data|
                @dont_remind = data 
                puts data
            }
                
            FXHorizontalSeparator.new(@content,SEPARATOR_GROOVE|LAYOUT_TOP|
                LAYOUT_LEFT|LAYOUT_FILL_X)                
        end
        
        def process_reminding()
            if @dont_remind
                @config["do_not_remind_about_" + @tool] = true
            end
        end
    end
    
end


class String
    def word_split(limit)
        current = self
        res = []
        while true
            cut, rest = current.cut_chunk(limit)
            res << cut
            current = rest
            return res if rest.nil?
        end
    end

    protected

    def cut_chunk(limit)
        return self, nil if self.length < limit
        limit.downto(0) do |i|
            if cut_possible(i)
                return self[0..i], self[i+1..-1]
            end
        end
        limit.upto(self.length) do |i|
            if cut_possible(i)
                return self[0..i], self[i+1..-1]
            end
        end


        return self, nil
    end

    def cut_possible(offset)
        pre_offset, offset = offset - 0, offset + 1

        if self[offset..offset] == " " or  self[offset..offset] == "\t"  or
                self[pre_offset..pre_offset] == " " or  self[pre_offset..pre_offset] == "\t"
            true
        else
            false
        end
    end
end

